'''
Created on 28/04/2009

Can handle boundary-intersection with a faster method, e.g., using rectangles!

@author: ben
'''

import pyglet
from euclid import *
import pickle
from misc import *
from TextRenderer import (TextRenderer, TextTransform)
from things import StaticThing
import globals

class InteractionObject(object):    
    def __init__(self,ia):
        self.ia = ia
        self.id = self.ia.id
        self.inside = False
        self.enteredBefore = False
    
    def position(self):
        return Point2(self.ia.x,self.ia.y)
    
    def str(self):
        return self.ia.id
        
    def entered(self):
        #print " entered ", self.str()
        self.inside = True     
        self.enteredBefore = True   
    
    def exitted(self):
        #print "exitted" , self.str()
        self.inside = False
        
    def used(self,item):
        #print "used ",item,"on",self.str()
        pass
        
    # helper functions
    def contains(self,(x,y)):
        #print "contains",x,y,self.ia.x,self.ia.y,self.ia.r
        if ((Vector2(x,y) - Vector2(self.ia.x,self.ia.y)).magnitude() < self.ia.r): return True
        else: return False

class Portal(InteractionObject):
    def __init__(self, room, ia, id):
        super(Portal,self).__init__(ia)
        self.room = room
        self.id = id
        self.connectedTo = None
        self.disabled = False
    
    def str(self):
        return "Portal(" + self.room.str() + ":" + str(self.id) + ")"
    
    def entered(self,who):
        super(Portal,self).entered()
        #TODO: emit a portal entered signal to the gameoverlord?
        #print who, " entered ", self.str()
        
    
    
    def connect(self, portal):
        self.connectedTo = portal

class Room(object):
    '''
    A room in the house, also includes outside areas, etc..
    Each room is a singleton and is responsible for its initialisation.
    '''
    
    @staticmethod
    def BlankRoom(name,mapdata):
        "create an empty room, that has walk info, portal info and staticimage objets and that's about it"
        r = Room(name,mapdata)
        objects = {}
        for obj in r.mapData.objects:
            objects[obj.name] = StaticThing(obj.img)        
        r.loadAndSortObjects(objects)
        io = {}
        for iz in r.mapData.interactionAreas:
            try:
                # don't add numbers/portals
                int(iz.id)
            except ValueError:            
                io[iz.id] = InteractionObject
        r.setInteractionZones(io)
        return r    
    
    def __init__(self,name,mapdata):
        "every room loads its mapdata from a file"
        self.name = name
        mapDataFile = pyglet.resource.file(mapdata,mode='r') 
        assert mapDataFile is not None
        #if __debug__:
        #    print "Loading ", mapdata
        self.mapData = pickle.load(mapDataFile)    
        self.bgimage = pyglet.resource.image(self.mapData.bgimage)
        self.width = self.bgimage.width
        self.height = self.bgimage.height
        self.texttransform = self.mapData.textAreaTransformation
    
    def str(self):
        return self.name
    
    def loadAndSortObjects(self,objects={}):
        "sorts the objects by y-pos and layer"
        
        # compile object list, load media, scripts, etc..
        # objects[i][j] gives the object j sitting on layer i
        #self.objects = {}
        self.objects = []
        #self.allobjects = []
        allObjects = self.mapData.objects
        for obj in allObjects:
            #layer = objlayers[obj.name]
            #self.objects[layer] = []
            
            
            # objects contains the special thing information
            # but we still need to add position and pivot information
            thing = objects[obj.name]
            thing.name = obj.name
            thing.room = self
            thing.position = Point2(obj.position[0] + obj.pivot[0], obj.position[1] + obj.pivot[1])
            thing.pivot = Point2(obj.pivot[0],obj.pivot[1])
            thing.postinit()
            
            # sort them
            for i in range(len(self.objects)):
                oy = self.objects[i].position[1]
                if thing.position[1]>oy:
                    self.objects.insert(i,thing)
                    break
            #if thing not in self.objects:        
            #    self.objects[objlayers[obj.name]].append(thing)
                
            if thing not in self.objects: 
                self.objects.append(thing)
            
        #self.sortedLayerIndexes = sorted(self.objects.keys())        
    
    def setInteractionZones(self,ios={}):
        self.interactionObjects = []
        self.portals = []
        #if __debug__:
        #    print "Loading interaction areas and portals for ", self.name
        for ia in self.mapData.interactionAreas:
            #if __debug__:
            #    print "ia (", ia.id, ")"
            if ia.id in ios:
                self.interactionObjects.append(ios[ia.id](ia))
            else:
                try:
                    portalNumber = int(ia.id)
                    #if __debug__:
                    #    print "found portal ", portalNumber
                except ValueError:
                    #print ("Unknown interaction zone: ",ia.id)
                    assert False
                else:
                    self.portals.append(Portal(self,ia,portalNumber))
                
    def activatePortal(self,character,portal):
        "push character into this portal, doesn't update entered/exitted etc.."
        character.room = portal.connectedTo.room
        character.position = portal.connectedTo.position()        
    
    def activatePortalPlayer(self,portal):
        self.activatePortal(globals.player,portal)
        portal.exitted()
        portal.connectedTo.entered(globals.player)
        globals.game.playerChangedRoom()
     
    def moveto(self,player,(x,y)):
        "The player has moved to this location, handle portals, and update the event handlers"
        # first check portals
        for portal in self.portals:
            contains = portal.contains((x,y))
            if portal.disabled: continue
            # else            
            if portal.inside and not contains:
                portal.exitted()
            elif not portal.inside and contains:
                portal.entered(player)
                if portal.connectedTo:
                    # this is an out portal, so do the portal jump
                    self.activatePortalPlayer(portal)
                                                
        # then check interactionObjects       
        for io in self.interactionObjects:
            contains = io.contains((x,y))
            if io.inside and not contains:
                io.exitted()
            elif not io.inside and contains:
                io.entered()
    
    def use(self,dir=None,item=None):
        "Given a facing direction and item, pass this event to the ios"
        for io in self.interactionObjects:
            if io.inside:
                io.used(item)
    
    def canMoveTo(self,p1,p2):
        "is it possible to move from p1 to p2"
        return not poly_intersects(self.mapData.walkArea,p1,p2)
    
    def adjacentTo(self,room):
        for p in self.portals:
            if p.connectedTo is not None and p.connectedTo.room==room: return True
        return False
    
    def getObject(self,name):
        for o in self.objects:
            if o.name==name: return o
        return None
    
    def getInteractionObject(self,name):
        for io in self.interactionObjects:
            if io.id==name: return io
        return None
        
    def getPortal(self,i):
        for io in self.portals:
            if io.id==i:
                return io
        return None
        
              