'''
Created on 30/04/2009

@author: ben
'''

import pyglet
from pyglet.gl import *
import pyglet.window.key as KEY
from random import random
from euclid import *
import math, os

from TextRenderer import (TextRenderer, TextTransform)
from characters import TheMan

#from Rooms import Kitchen
from room import Room
import globals

class RoomRenderer:
    '''
    Offers static functionality for rendering a room with inhabitants, text, etc.
    '''

    # debug draws debugging information
    debug = False

    room = None
    textrenderer = None
    hidetext = True
    kookyMode = False
    initialised = False
    x = 0
    y = 0
    
    @staticmethod
    def resize():
        RoomRenderer.x = (globals.game.width - RoomRenderer.room.width)/2
        RoomRenderer.y = (globals.game.height - RoomRenderer.room.height)/2
        
    @staticmethod
    def setRoom(room):
        "Sets a new room to render, sets defaults"
        if room is not RoomRenderer.room:
            RoomRenderer.room = room
            if not RoomRenderer.initialised:
                  RoomRenderer.initialised = True
                  RoomRenderer.textrenderer = TextRenderer()
                  RoomRenderer.textrenderer.setText("")
            
            RoomRenderer.x = (globals.game.width - room.width)/2
            RoomRenderer.y = (globals.game.height - room.height)/2
            RoomRenderer.textrenderer.setTransform(RoomRenderer.room.texttransform)
            #RoomRenderer.hidetext = True
    
    @staticmethod
    def setText(text,size=6):
        "Show some text in the room, or pass None to disable text display"
        if text==None:
            RoomRenderer.hidetext = True
        else:
            RoomRenderer.hidetext = False
            RoomRenderer.textrenderer.setText(text,size)
    
    @staticmethod
    def render(characters=[],objects=[]):
        "render using the setup variables"
        RoomRenderer.render_specific(RoomRenderer.room,characters,objects,RoomRenderer.textrenderer if not RoomRenderer.hidetext else None)
    
    @staticmethod
    def begin():
        "begin draw"
        #TODO:
        # setup viewport, place room in correct position
        glPushAttrib(GL_VIEWPORT_BIT)
        glViewport(RoomRenderer.x,RoomRenderer.y,RoomRenderer.room.width,RoomRenderer.room.height)
                
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        gluOrtho2D(0,RoomRenderer.room.width,0,RoomRenderer.room.height)
        glMatrixMode(GL_MODELVIEW)       
      
    @staticmethod  
    def end():
        "end draw"
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glPopAttrib()
    
    @staticmethod
    def render_specific(room,characters=[],objects=[],textrenderer=None):
        "render a room, characters, text"         
        
        RoomRenderer.begin()
        
        # blit background
        glColor3f(1,1,1)
        room.bgimage.blit(0,0) 
        
        RoomRenderer.end()
        
        # a hack, but .. whatever...!
        
        glPushAttrib(GL_VIEWPORT_BIT)
        glViewport(RoomRenderer.x,RoomRenderer.y,800,600)
        if textrenderer:
            textrenderer.draw(room.width,room.height)
        glPopAttrib()
        
        RoomRenderer.begin()
        
        # collect all the characters in this room
        chs = [ch for ch in characters if ch.room==room]
        for ch in chs:
            if RoomRenderer.kookyMode:
                ch.moveBy(2*random()-1,2*random()-1)
            ch.drawnThisFrame = False
        
        # draw the objects from back to front
        # from low layer to high
        for o in room.objects+objects:
            if RoomRenderer.kookyMode:
                o.moveBy(2*random()-1,2*random()-1)
            
            for ch in [ch for ch in chs if not ch.drawnThisFrame]:
                if o.y() < ch.y():
                    ch.draw()
                    ch.drawnThisFrame = True
            o.draw()                
        
        for ch in [ch for ch in chs if not ch.drawnThisFrame]:
            ch.draw()        
        
        if RoomRenderer.debug:
            #dbg draw
            # walkarea
            md = room.mapData
            if md.walkArea:
                glColor3f(1,0,0)
                glLineWidth(2)
                glBegin(GL_LINE_LOOP)
                for x,y in md.walkArea:
                    glVertex2f(x,y)
                glEnd()   
        
        RoomRenderer.end()
        

# room renderer test
if __name__ == '__main__':
    
    #from World import World

    # set up resource info
    pyglet.resource.path = ['../res']
    pyglet.resource.reindex()

    width, height = 800, 600
    window = pyglet.window.Window(width, height, visible=False)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    
    #world = World()
    #player = world.player
    room = Room.BlankRoom("room","front.dat")
    theman = TheMan(room,Point2(room.width/2,room.height/2))
    
    RoomRenderer.setRoom(room)
    RoomRenderer.debug = True
    
    fps_display = pyglet.clock.ClockDisplay()
    
    change_text_counter = 0
    def change_text(dt):
        global text, change_text_counter
        
        text = ['''Get off my LAWN!'''.upper(),
                    '''A short game by Ben Porter.'''.upper(),
                    '''Enjoy!'''.upper()][change_text_counter]
        RoomRenderer.setText(text)
        change_text_counter += 1
        if change_text_counter >= 3: change_text_counter = 2
            
            
    # 
    t = 0
    display_update = 1/60.0
    
    @window.event
    def on_draw():
        global t
        t += display_update
        theman.localUpdate(display_update)
        
        window.clear()  
        
        RoomRenderer.render([theman])
        
        fps_display.draw()

    pyglet.clock.schedule_interval(change_text, 2)
    pyglet.clock.schedule_interval(lambda dt: None, display_update)
    window.set_visible(True)    
    pyglet.app.run()  