'''
Created on 27/04/2009

Load a scene and test a walking character

@author: ben
'''

import pyglet
from pyglet.gl import *
from euclid import *
from copy import copy
import pyglet.window.key as KEY
import math, os
from random import random
from MapData import *
import pickle
from misc import *
from Room import *
from World import World

if __name__ == '__main__':
    # set up resource info
    pyglet.resource.path = ['.','../data','../data/mftm-ambient']
    pyglet.resource.reindex()

    width, height = 800, 600
    window = pyglet.window.Window(width, height, visible=False)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    
    world = World()
    player = world.player
    
    debugdraw = False
    music = False    
    
    if music:
        musicPlayer = pyglet.media.Player()
        musicPlayer.eos_action = pyglet.media.Player.EOS_LOOP
        
        try:
            music = pyglet.resource.media("01 - Ambient A 13.47.mp3")
            sfx = pyglet.resource.media("64346__Robinhood76__00672_gunshot_1.wav",streaming=False)
            musicPlayer.queue(music)
            musicPlayer.play()
        except IOError:
            print("Couldn't find audio file")
            music = None
        except pyglet.media.MediaFormatException:
            print("Media Format Exception!")
            music = None
    
    fps_display = pyglet.clock.ClockDisplay()

    
    # 
    t = 0
    display_update = 1/60.0
    world_local_update = display_update
    world_global_update = 1/10.

    mx, my = 0, 0
    @window.event
    def on_mouse_motion(x,y,dx,dy):
        global mx,my
        mx,my = x,y
        
    @window.event
    def on_mouse_press(x,y,button,mod):
        print ("mouse click: ", (x,y))
            
    @window.event
    def on_key_press(key,mod):
        global debugdraw
        if key==KEY.D:
            debugdraw = not debugdraw
        if key==KEY.SPACE:
            player.room.use()
        else:
            walkingDirection = Dir.dirFromKey(key)
            if walkingDirection==None:
                player.stopWalking()
            else:
                player.startWalking(walkingDirection)
            
    @window.event
    def on_key_release(key,mod):
        if player.isWalking and player.walkingDirection==Dir.dirFromKey(key):
            player.stopWalking()  
    
    def update(dt):
        world.localUpdate(dt)
        
    def globalUpdate(dt):
        world.globalUpdate(dt)
        
    # test Room.canMoveTo
    cmtlines = []
    for i in range(1000):
        p1,p2 = Point2(random()*width,random()*height),Point2(random()*width,random()*height)
        if player.room.canMoveTo(p1,p2):
            cmtlines.append((p1,p2))
                        
    @window.event
    def on_draw():
        global t
        t += display_update
        
        window.clear()   
        glColor3f(1,1,1)
        player.room.bgimage.blit(0,0) 
        
        layerindex = world.player.room.mapData.inLayer(player.position)
        
        # collect all the characters in this room
        chs = [ch for ch in world.characters if ch.room==player.room]
        for ch in chs:
            ch.drawnThisFrame = False
            ch.li = player.room.mapData.inLayer(ch.position)
        
        # draw the objects from back to front
        # from low layer to high
        for li in world.player.room.sortedLayerIndexes:
            for o in world.player.room.objects[li]:
                for ch in [ch for ch in chs if not ch.drawnThisFrame]:
                    if ch.li==li and o.y() < ch.y():
                        ch.draw()
                        ch.drawnThisFrame = True
                o.draw()                
        
        for ch in [ch for ch in chs if not ch.drawnThisFrame]:
            ch.draw()        
        
        if debugdraw:
            #dbg draw
            # walkarea
            md = player.room.mapData
            if md.walkArea:
                glColor3f(1,0,0)
                glLineWidth(2)
                glBegin(GL_LINE_LOOP)
                for x,y in md.walkArea:
                    glVertex2f(x,y)
                glEnd()        
                
            # draw wall collision        
            l = poly_intersects_last_line
            if l:
                glLineWidth(2)
                glColor3f(1,0,0)
                glBegin(GL_LINES)
                glVertex2f(l.p1.x,l.p1.y)
                glVertex2f(l.p2.x,l.p2.y)
                glEnd()
            
            # draw canmoveto line test
            glColor4f(0,0,0,.05)
            glLineWidth(1)
            glBegin(GL_LINES)
            for (p1,p2) in cmtlines:  
                glVertex2f(p1.x,p1.y)
                glVertex2f(p2.x,p2.y)
            glEnd() 
            glColor4f(1,1,1,1)   

        fps_display.draw()

    pyglet.clock.schedule_interval(lambda dt: None, display_update)
    pyglet.clock.schedule_interval(update, world_local_update)
    pyglet.clock.schedule_interval(globalUpdate, world_global_update)
    window.set_visible(True)    
    pyglet.app.run()  
    