'''
Created on 28/04/2009

@author: ben
'''
import pyglet
from euclid import *

class Thing(object):
    '''
    A thing exists in the world. It is a character or static object, and can be updated, has state, etc.
    '''

    def __init__(self):
        self.name = None
        self.position = None # a Point2
        self.room = None
        self.pivot = None # a Point2 (used to position the sprites correctly)        
        self.globalUpdateNeeded = False
        self.localUpdateNeeded = False
        
    def moveBy(self,dx,dy):
        self.position.x += dx
        self.position.y += dy
        
    def postinit(self):
        "this is sometimes called after the object vars have been set"
        pass
    
    def y(self):
        "retrieve the y position, used to depth sort"
        return self.position.y
    
    def draw(self):
        "assume that draw is only executed when the room is being viewed"
        pass
    
    def localUpdate(self,dt):
        "called frequently, but only when the object is in view"
        pass
    
    def globalUpdate(self,dt):
        "use this to change things whether or not the thing is in view"
        pass
    
    def notify(self,str):
        "send a message to this thing"
        pass
    
    @staticmethod
    def distance(thing1,thing2):
        "distance between two things"
        return (Vector2(thing1.position.x,thing1.position.y) - Vector2(thing2.position.x,thing2.position.y)).magnitude()

class StaticThing(Thing):
    '''
    A static image
    '''
    
    def __init__(self,imgname):
        super(StaticThing,self).__init__()
        self.img = pyglet.resource.image(imgname)
        self.show = True
    
    def postinit(self):
        if self.pivot:
            self.img.anchor_x,self.img.anchor_y = self.pivot.x,self.pivot.y
            
    def draw(self):
        if self.show:
            self.img.blit(self.position.x,self.position.y)
        
        
        