'''
Created on 27/04/2009

Load a scene and test a walking character

@author: ben
'''

import pyglet
from pyglet.gl import *
from euclid import *
from copy import copy
import pyglet.window.key as KEY
import math, os
from random import random
from MapData import *
import pickle
from misc import *

class InteractionObject(object):    
    def __init__(self,ia):
        self.ia = ia
        self.inside = False
        self.enteredBefore = False
    
    def str(self):
        return self.ia.id
        
    def entered(self):
        print ("entered",self.str())
        self.inside = True     
        self.enteredBefore = True   
    
    def exitted(self):
        print ("exitted" ,self.str())
        self.inside = False
        
    def used(self,item):
        print ("used ",item,"on",self.str())
        
    # helper functions
    def contains(self,(x,y)):
        if (Point2(x,y).distance(Point2(self.ia.x,self.ia.y)) < self.ia.r): return True
        else: return False

class KitchenWindow(InteractionObject):
    def entered(self):
        InteractionObject.entered(self)
        if not self.enteredBefore:
            print "It's a nice day outside"
        else:
            print "Looking out the window makes me depressed"
        
class KitchenFridge(InteractionObject):
    def __init__(self,ia):
        super(KitchenFridge,self).__init__(ia)
        
    def used(self,item):
        InteractionObject.used(self,item)
        if item==None:
            print "Opening fridge"
        else:
            print "I can't use that here!"

class Kitchen(object):
    '''
    Each room will have its own singleton class - for ease of set up, etc.
    '''
    def __init__(self):
        self.mapData = pickle.load(open("p.pickle","r"))
        
        # compile object list, load media, scripts, etc..
        # objects[i][j] gives the object j sitting on layer i
        objlayers = {'fridge':1,'bench':0}
        objimages = {'fridge':'fridge.png','bench':'bench.png'}
        self.objects = {}
        self.allobjects = []
        allObjects = self.mapData.objects
        for obj in allObjects:
            self.objects[objlayers[obj.name]] = []
            newobj = Bunch(name=obj.name,position=obj.position,pivot=obj.pivot,img=objimages[obj.name])
            self.objects[objlayers[obj.name]].append(newobj)
            self.allobjects.append(newobj)
            
            # would need to sort them if there were more than one per layer
        self.sortedLayerIndexes = sorted(self.objects.keys())
        
        # interaction zones
        # map from interactionzones to interaction objects
        ios = {'window':KitchenWindow,'fridge':KitchenFridge}
        
        self.interactionObjects = []
        for ia in self.mapData.interactionAreas:
            self.interactionObjects.append(ios[ia.id](ia))
            
    def moveto(self,(x,y)):
        "The player has moved to this location, so update the event handlers"
        for io in self.interactionObjects:
            contains = io.contains((x,y))
            if io.inside and not contains:
                io.exitted()
            elif not io.inside and contains:
                io.entered()        
    
    def use(self,dir=None,item=None):
        "Given a facing direction and item, pass this event to the ios"
        for io in self.interactionObjects:
            if io.inside:
                io.used(item)
                   

if __name__ == '__main__':
    # set up resource info
    pyglet.resource.path = ['../data']
    pyglet.resource.reindex()

    width, height = 800, 600
    window = pyglet.window.Window(width, height, visible=False)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    
    kitchen = Kitchen()
    md = kitchen.mapData
    
    bgimage = pyglet.resource.image(md.bgimage)
    objimages = {}    
    for obj in kitchen.allobjects:
        objimages[obj.img] = pyglet.resource.image(obj.img)
    
    walker = pyglet.resource.image("circle.png")
    walker.anchor_x = walker.width/2
    # polygon bounding the walkable area
    # anticlockwise around the walkable area
    #md.walkArea = [(316, 338), (131, 337), (71, 73), (736, 81), (651, 336), (397, 336), (457, 187), (373, 185)] 
        
    # walk state information
    UP = 0
    DOWN = 1
    
    isWalking = False
    walkingDirection = None
    position = (100,100)
    walkerWidth = walker.width
    walkingSpeed = 200 #pixels per seoncd
    lastWalkLine = None
    lineclash = None
    
    
    def newPosition((x,y)):
        "update the position of the character"
        global position
        position = (x,y)
        kitchen.moveto(position)
    
    newPosition(position)
    
    # 
    t = 0
    dt = 1/60.0

    mx, my = 0, 0
    @window.event
    def on_mouse_motion(x,y,dx,dy):
        global mx,my
        mx,my = x,y
        
    @window.event
    def on_mouse_press(x,y,button,mod):
        pass
    
    @window.event
    def on_key_press(key,mod):
        global isWalking, walkingDirection
        if key==KEY.SPACE:
            kitchen.use()
        else:
            walkingDirection = Dir.dirFromKey(key)
            if walkingDirection==None:
                isWalking = False
                #print("stop walking")
            elif not isWalking:
                isWalking = True
                #print("start walking")
            
    @window.event
    def on_key_release(key,mod):
        global isWalking
        if isWalking and walkingDirection==Dir.dirFromKey(key):
            isWalking = False
            #print("stop walking")
    
        
    
    def update(dt):
        global isWalking, position, lineclash, lastWalkLine
        if isWalking:
            dx = {Dir.DOWN:0,Dir.UP:0,Dir.LEFT:-1,Dir.RIGHT:1}[walkingDirection]
            dy = {Dir.DOWN:-1,Dir.UP:1,Dir.LEFT:0,Dir.RIGHT:0}[walkingDirection]
            oldpos = Point2(position[0],position[1])
            dpos = Point2(dx,dy)*(walkingSpeed*dt)
            newpos = oldpos + dpos
            # test if this position is reachable
            isReachable = True
            # the moveline has to be from the leading point, or from both if up/down
            startPosRel = Point2(0,0)
            if (walkingDirection!=Dir.DOWN and walkingDirection!=Dir.UP):
                if (dx>0): startPosRel = Point2(walkerWidth/2,0)
                else: startPosRel = Point2(-walkerWidth/2,0)
                            
                moveLine = LineSegment2(oldpos,dpos+startPosRel)
                lastWalkLine = moveLine
                for i in range(len(md.walkArea)):
                    ni = (i + 1)%len(md.walkArea)
                    p1 = apply(Point2,md.walkArea[i])
                    p2 = apply(Point2,md.walkArea[ni])
                    borderLine = LineSegment2(p1,p2)
                    if moveLine.intersect(borderLine):
                        isReachable = False
                        lineclash = i
                        break
            else:
                moveLine1 = LineSegment2(oldpos,dpos+Point2(walkerWidth/2,0))
                moveLine2 = LineSegment2(oldpos,dpos+Point2(-walkerWidth/2,0))
                lastWalkLine = LineSegment2(oldpos,dpos)
                for i in range(len(md.walkArea)):
                    ni = (i + 1)%len(md.walkArea)
                    p1 = apply(Point2,md.walkArea[i])
                    p2 = apply(Point2,md.walkArea[ni])
                    borderLine = LineSegment2(p1,p2)
                    if moveLine1.intersect(borderLine) or moveLine2.intersect(borderLine):
                        isReachable = False
                        lineclash = i
                        break
            
            if isReachable:                
                newPosition((newpos.x,newpos.y))
                lineclash = None
            else:
                isWalking = False
                
    # generate line segments, and test for intersection against any linesegments in walk area
    linesegsout = []
    for i in range(1000):
        p1 = Point2(random()*width,random()*height)
        p2 = Point2(random()*width,random()*height)
        line = LineSegment2(p1,p2)
        intersects = False
        for i in range(len(md.walkArea)):
            ni = (i + 1)%len(md.walkArea)
            p1 = apply(Point2,md.walkArea[i])
            p2 = apply(Point2,md.walkArea[ni])
            borderLine = LineSegment2(p1,p2)
            if line.intersect(borderLine):
                intersects = True
                break
        if not intersects:
           linesegsout.append(line)
                
    @window.event
    def on_draw():
        global t
        t += dt
        
        window.clear()   
        glColor3f(1,1,1)   
        bgimage.blit(0,0) 
        
        layerindex = kitchen.mapData.inLayer(position)
        drawnWalker = False
        
        # draw the objects from back to front
        # from low layer to high
        for li in kitchen.sortedLayerIndexes:
            for o in kitchen.objects[li]:
                if not drawnWalker and layerindex==li and o.pivot[1] + o.position[1] < position[1]:
                    drawnWalker = True
                    apply(walker.blit,position)
                img = objimages[o.img]
                apply(img.blit,o.position)
        if not drawnWalker:
            drawnWalker = True
            apply(walker.blit,position)
        # TODO: Sort objects (only need to do this once!)
        # TODO: Fix layering system, see bench!
        
        # line clash
        if lineclash:
            glColor3f(0,0,1)
            glLineWidth(4)
            glBegin(GL_LINES)
            apply(glVertex2f,md.walkArea[lineclash])
            apply(glVertex2f,md.walkArea[(lineclash+1)%len(md.walkArea)])
            glEnd()
            
        if lastWalkLine:
            glLineWidth(4)
            glColor3f(0,1,1)
            glBegin(GL_LINES)
            glVertex2f(lastWalkLine.p1.x,lastWalkLine.p1.y)
            glVertex2f(lastWalkLine.p2.x,lastWalkLine.p2.y)
            glEnd()
            
#===============================================================================
#        glLineWidth(1)
#        glColor3f(1,0,0)
#        glBegin(GL_LINES)
#        for l in linesegsout:
#            glVertex2f(l.p1.x,l.p1.y)
#            glVertex2f(l.p2.x,l.p2.y)
#        glEnd()
#===============================================================================

    pyglet.clock.schedule_interval(lambda dt: None, dt)
    pyglet.clock.schedule_interval(update, dt)
    window.set_visible(True)    
    pyglet.app.run()  
    